[INFRA] Import NVIDIA/CCCL upstream as optimization reference library
CCCL (CUDA C++ Core Libraries) provides: - CUB: device/block/warp-level GPU primitives (reduce, scan, sort, topk) - Thrust: high-level parallel algorithms (transform_reduce, sort, scan) - libcudacxx: CUDA C++ standard library (atomics, barriers, memory) - cudax: experimental features (memory resources, allocators) - Tuning policies: per-SM hardware-specific algorithm parameters Competition optimization vectors mapped to CCCL: - Output TPS (83% weight): warp_reduce, block_reduce, device_topk - Input TPS (14% weight): device_scan, block_load, prefetch - Cache TPS (3% weight): prefix caching strategy patterns - Memory (0.9 util): pooled/cached/buddy allocators Source: https://github.com/NVIDIA/cccl (shallow clone, HEAD only) License: Apache-2.0
This commit is contained in:
115
cccl_upstream/cudax/test/stf/threads/axpy-threads-2.cu
Normal file
115
cccl_upstream/cudax/test/stf/threads/axpy-threads-2.cu
Normal file
@@ -0,0 +1,115 @@
|
||||
//===----------------------------------------------------------------------===//
|
||||
//
|
||||
// Part of CUDASTF in CUDA C++ Core Libraries,
|
||||
// under the Apache License v2.0 with LLVM Exceptions.
|
||||
// See https://llvm.org/LICENSE.txt for license information.
|
||||
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES.
|
||||
//
|
||||
//===----------------------------------------------------------------------===//
|
||||
|
||||
/**
|
||||
* @file
|
||||
*
|
||||
* @brief An AXPY kernel implemented with a task of the CUDA stream backend
|
||||
*
|
||||
*/
|
||||
|
||||
#include <cuda/experimental/__stf/stream/stream_ctx.cuh>
|
||||
|
||||
#include <mutex>
|
||||
#include <thread>
|
||||
|
||||
using namespace cuda::experimental::stf;
|
||||
|
||||
static __global__ void cuda_sleep_kernel(long long int clock_cnt)
|
||||
{
|
||||
long long int start_clock = clock64();
|
||||
long long int clock_offset = 0;
|
||||
while (clock_offset < clock_cnt)
|
||||
{
|
||||
clock_offset = clock64() - start_clock;
|
||||
}
|
||||
}
|
||||
|
||||
void cuda_sleep(double ms, cudaStream_t stream)
|
||||
{
|
||||
int device;
|
||||
cudaGetDevice(&device);
|
||||
|
||||
// cudaDevAttrClockRate: Peak clock frequency in kilohertz;
|
||||
int clock_rate;
|
||||
cudaDeviceGetAttribute(&clock_rate, cudaDevAttrClockRate, device);
|
||||
|
||||
long long int clock_cnt = (long long int) (ms * clock_rate);
|
||||
cuda_sleep_kernel<<<1, 1, 0, stream>>>(clock_cnt);
|
||||
}
|
||||
|
||||
__global__ void axpy(double a, slice<const double> x, slice<double> y)
|
||||
{
|
||||
int tid = blockIdx.x * blockDim.x + threadIdx.x;
|
||||
int nthreads = gridDim.x * blockDim.x;
|
||||
|
||||
for (int i = tid; i < x.size(); i += nthreads)
|
||||
{
|
||||
y(i) += a * x(i);
|
||||
}
|
||||
}
|
||||
|
||||
double X0(int i)
|
||||
{
|
||||
return sin((double) i);
|
||||
}
|
||||
|
||||
double Y0(int i)
|
||||
{
|
||||
return cos((double) i);
|
||||
}
|
||||
|
||||
void mytask(stream_ctx ctx, int /*id*/, logical_data<slice<double>> lX)
|
||||
{
|
||||
// std::cout << "Thread " << id << " is executing.\n";
|
||||
|
||||
const size_t N = 16;
|
||||
|
||||
double alpha = 3.14;
|
||||
|
||||
auto lY = ctx.logical_data<double>(N);
|
||||
ctx.task(lY.write())->*[](cudaStream_t, auto) {};
|
||||
|
||||
/* Compute Y = Y + alpha X */
|
||||
for (size_t i = 0; i < 10; i++)
|
||||
{
|
||||
ctx.task(lX.read(), lY.rw())->*[&](cudaStream_t s, auto dX, auto dY) {
|
||||
axpy<<<16, 128, 0, s>>>(alpha, dX, dY);
|
||||
cuda_sleep(100.0, s);
|
||||
};
|
||||
}
|
||||
}
|
||||
|
||||
int main()
|
||||
{
|
||||
stream_ctx ctx;
|
||||
const size_t N = 16;
|
||||
auto lX = ctx.logical_data<double>(N);
|
||||
ctx.task(lX.write())->*[](cudaStream_t, auto) {};
|
||||
|
||||
std::vector<std::thread> threads;
|
||||
// Launch 8 threads.
|
||||
for (int i = 0; i < 10; ++i)
|
||||
{
|
||||
threads.emplace_back(mytask, ctx, i, lX);
|
||||
}
|
||||
|
||||
// Wait for all threads to complete.
|
||||
for (auto& th : threads)
|
||||
{
|
||||
th.join();
|
||||
}
|
||||
|
||||
ctx.task(lX.rw())->*[&](cudaStream_t s, auto) {
|
||||
cuda_sleep(100.0, s);
|
||||
};
|
||||
|
||||
ctx.finalize();
|
||||
}
|
||||
@@ -0,0 +1,85 @@
|
||||
//===----------------------------------------------------------------------===//
|
||||
//
|
||||
// Part of CUDASTF in CUDA C++ Core Libraries,
|
||||
// under the Apache License v2.0 with LLVM Exceptions.
|
||||
// See https://llvm.org/LICENSE.txt for license information.
|
||||
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2022-2025 NVIDIA CORPORATION & AFFILIATES.
|
||||
//
|
||||
//===----------------------------------------------------------------------===//
|
||||
|
||||
/**
|
||||
* @file
|
||||
*
|
||||
* @brief Ensure a graph_ctx can be used concurrently
|
||||
*
|
||||
*/
|
||||
|
||||
#include <cuda/experimental/stf.cuh>
|
||||
|
||||
#include <mutex>
|
||||
#include <thread>
|
||||
|
||||
using namespace cuda::experimental::stf;
|
||||
|
||||
__global__ void axpy(slice<int> y, slice<const int> x, int a)
|
||||
{
|
||||
int tid = blockIdx.x * blockDim.x + threadIdx.x;
|
||||
int nthreads = gridDim.x * blockDim.x;
|
||||
|
||||
for (int i = tid; i < x.size(); i += nthreads)
|
||||
{
|
||||
y(i) += a * x(i);
|
||||
}
|
||||
}
|
||||
|
||||
void mytask(graph_ctx ctx, int /*id*/)
|
||||
{
|
||||
const size_t N = 16;
|
||||
|
||||
int alpha = 3;
|
||||
|
||||
auto lX = ctx.logical_data<int>(N);
|
||||
auto lY = ctx.logical_data<int>(N);
|
||||
|
||||
ctx.parallel_for(lX.shape(), lX.write(), lY.write())->*[] __device__(size_t i, auto x, auto y) {
|
||||
x(i) = (1 + i);
|
||||
y(i) = (2 + i * i);
|
||||
};
|
||||
|
||||
/* Compute Y = Y + alpha X */
|
||||
for (size_t i = 0; i < 200; i++)
|
||||
{
|
||||
ctx.task(lY.rw(), lX.read())->*[alpha](cudaStream_t stream, auto dY, auto dX) {
|
||||
axpy<<<128, 64, 0, stream>>>(dY, dX, alpha);
|
||||
};
|
||||
}
|
||||
|
||||
ctx.host_launch(lX.read(), lY.read())->*[alpha](auto x, auto y) {
|
||||
for (size_t i = 0; i < N; i++)
|
||||
{
|
||||
EXPECT(x(i) == 1 + i);
|
||||
EXPECT(y(i) == 2 + i * i + 200 * alpha * x(i));
|
||||
}
|
||||
};
|
||||
}
|
||||
|
||||
int main()
|
||||
{
|
||||
graph_ctx ctx;
|
||||
|
||||
::std::vector<::std::thread> threads;
|
||||
// Launch threads
|
||||
for (int i = 0; i < 10; ++i)
|
||||
{
|
||||
threads.emplace_back(mytask, ctx, i);
|
||||
}
|
||||
|
||||
// Wait for all threads to complete.
|
||||
for (auto& th : threads)
|
||||
{
|
||||
th.join();
|
||||
}
|
||||
|
||||
ctx.finalize();
|
||||
}
|
||||
74
cccl_upstream/cudax/test/stf/threads/axpy-threads-graph.cu
Normal file
74
cccl_upstream/cudax/test/stf/threads/axpy-threads-graph.cu
Normal file
@@ -0,0 +1,74 @@
|
||||
//===----------------------------------------------------------------------===//
|
||||
//
|
||||
// Part of CUDASTF in CUDA C++ Core Libraries,
|
||||
// under the Apache License v2.0 with LLVM Exceptions.
|
||||
// See https://llvm.org/LICENSE.txt for license information.
|
||||
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2022-2025 NVIDIA CORPORATION & AFFILIATES.
|
||||
//
|
||||
//===----------------------------------------------------------------------===//
|
||||
|
||||
/**
|
||||
* @file
|
||||
*
|
||||
* @brief Ensure a graph_ctx can be used concurrently
|
||||
*
|
||||
*/
|
||||
|
||||
#include <cuda/experimental/stf.cuh>
|
||||
|
||||
#include <mutex>
|
||||
#include <thread>
|
||||
|
||||
using namespace cuda::experimental::stf;
|
||||
|
||||
void mytask(graph_ctx ctx, int /*id*/)
|
||||
{
|
||||
const size_t N = 16;
|
||||
|
||||
int alpha = 3;
|
||||
|
||||
auto lX = ctx.logical_data<int>(N);
|
||||
auto lY = ctx.logical_data<int>(N);
|
||||
|
||||
ctx.parallel_for(lX.shape(), lX.write(), lY.write())->*[] __device__(size_t i, auto x, auto y) {
|
||||
x(i) = (1 + i);
|
||||
y(i) = (2 + i * i);
|
||||
};
|
||||
|
||||
/* Compute Y = Y + alpha X */
|
||||
for (size_t i = 0; i < 200; i++)
|
||||
{
|
||||
ctx.parallel_for(lY.shape(), lY.rw(), lX.read())->*[alpha] __device__(size_t i, auto dY, auto dX) {
|
||||
dY(i) += alpha * dX(i);
|
||||
};
|
||||
}
|
||||
|
||||
ctx.host_launch(lX.read(), lY.read())->*[alpha](auto x, auto y) {
|
||||
for (size_t i = 0; i < N; i++)
|
||||
{
|
||||
EXPECT(x(i) == 1 + i);
|
||||
EXPECT(y(i) == 2 + i * i + 200 * alpha * x(i));
|
||||
}
|
||||
};
|
||||
}
|
||||
|
||||
int main()
|
||||
{
|
||||
graph_ctx ctx;
|
||||
|
||||
::std::vector<::std::thread> threads;
|
||||
// Launch threads
|
||||
for (int i = 0; i < 10; ++i)
|
||||
{
|
||||
threads.emplace_back(mytask, ctx, i);
|
||||
}
|
||||
|
||||
// Wait for all threads to complete.
|
||||
for (auto& th : threads)
|
||||
{
|
||||
th.join();
|
||||
}
|
||||
|
||||
ctx.finalize();
|
||||
}
|
||||
114
cccl_upstream/cudax/test/stf/threads/axpy-threads-pfor.cu
Normal file
114
cccl_upstream/cudax/test/stf/threads/axpy-threads-pfor.cu
Normal file
@@ -0,0 +1,114 @@
|
||||
//===----------------------------------------------------------------------===//
|
||||
//
|
||||
// Part of CUDASTF in CUDA C++ Core Libraries,
|
||||
// under the Apache License v2.0 with LLVM Exceptions.
|
||||
// See https://llvm.org/LICENSE.txt for license information.
|
||||
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES.
|
||||
//
|
||||
//===----------------------------------------------------------------------===//
|
||||
|
||||
/**
|
||||
* @file
|
||||
*
|
||||
* @brief An AXPY kernel implemented with a task of the CUDA stream backend
|
||||
*
|
||||
*/
|
||||
|
||||
#include <cuda/experimental/__stf/stream/stream_ctx.cuh>
|
||||
|
||||
#include <mutex>
|
||||
#include <thread>
|
||||
|
||||
using namespace cuda::experimental::stf;
|
||||
|
||||
static __global__ void cuda_sleep_kernel(long long int clock_cnt)
|
||||
{
|
||||
long long int start_clock = clock64();
|
||||
long long int clock_offset = 0;
|
||||
while (clock_offset < clock_cnt)
|
||||
{
|
||||
clock_offset = clock64() - start_clock;
|
||||
}
|
||||
}
|
||||
|
||||
void cuda_sleep(double ms, cudaStream_t stream)
|
||||
{
|
||||
int device;
|
||||
cudaGetDevice(&device);
|
||||
|
||||
// cudaDevAttrClockRate: Peak clock frequency in kilohertz;
|
||||
int clock_rate;
|
||||
cudaDeviceGetAttribute(&clock_rate, cudaDevAttrClockRate, device);
|
||||
|
||||
long long int clock_cnt = (long long int) (ms * clock_rate);
|
||||
cuda_sleep_kernel<<<1, 1, 0, stream>>>(clock_cnt);
|
||||
}
|
||||
|
||||
__global__ void axpy(double a, slice<const double> x, slice<double> y)
|
||||
{
|
||||
int tid = blockIdx.x * blockDim.x + threadIdx.x;
|
||||
int nthreads = gridDim.x * blockDim.x;
|
||||
|
||||
for (int i = tid; i < x.size(); i += nthreads)
|
||||
{
|
||||
y(i) += a * x(i);
|
||||
}
|
||||
}
|
||||
|
||||
double X0(int i)
|
||||
{
|
||||
return sin((double) i);
|
||||
}
|
||||
|
||||
double Y0(int i)
|
||||
{
|
||||
return cos((double) i);
|
||||
}
|
||||
|
||||
void mytask(stream_ctx ctx, int /*id*/)
|
||||
{
|
||||
// std::cout << "Thread " << id << " is executing.\n";
|
||||
|
||||
const size_t N = 16;
|
||||
|
||||
double alpha = 3.14;
|
||||
|
||||
auto lX = ctx.logical_data<double>(N);
|
||||
auto lY = ctx.logical_data<double>(N);
|
||||
|
||||
ctx.parallel_for(lX.shape(), lX.write())->*[] __device__(size_t i, auto x) {
|
||||
x(i) = 1.0;
|
||||
};
|
||||
|
||||
ctx.task(lY.write())->*[](cudaStream_t, auto) {};
|
||||
|
||||
/* Compute Y = Y + alpha X */
|
||||
for (size_t i = 0; i < 10; i++)
|
||||
{
|
||||
ctx.task(lX.read(), lY.rw())->*[&](cudaStream_t s, auto dX, auto dY) {
|
||||
axpy<<<16, 128, 0, s>>>(alpha, dX, dY);
|
||||
cuda_sleep(100.0, s);
|
||||
};
|
||||
}
|
||||
}
|
||||
|
||||
int main()
|
||||
{
|
||||
stream_ctx ctx;
|
||||
|
||||
std::vector<std::thread> threads;
|
||||
// Launch 8 threads.
|
||||
for (int i = 0; i < 10; ++i)
|
||||
{
|
||||
threads.emplace_back(mytask, ctx, i);
|
||||
}
|
||||
|
||||
// Wait for all threads to complete.
|
||||
for (auto& th : threads)
|
||||
{
|
||||
th.join();
|
||||
}
|
||||
|
||||
ctx.finalize();
|
||||
}
|
||||
112
cccl_upstream/cudax/test/stf/threads/axpy-threads.cu
Normal file
112
cccl_upstream/cudax/test/stf/threads/axpy-threads.cu
Normal file
@@ -0,0 +1,112 @@
|
||||
//===----------------------------------------------------------------------===//
|
||||
//
|
||||
// Part of CUDASTF in CUDA C++ Core Libraries,
|
||||
// under the Apache License v2.0 with LLVM Exceptions.
|
||||
// See https://llvm.org/LICENSE.txt for license information.
|
||||
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES.
|
||||
//
|
||||
//===----------------------------------------------------------------------===//
|
||||
|
||||
/**
|
||||
* @file
|
||||
*
|
||||
* @brief An AXPY kernel implemented with a task of the CUDA stream backend
|
||||
*
|
||||
*/
|
||||
|
||||
#include <cuda/experimental/__stf/stream/stream_ctx.cuh>
|
||||
|
||||
#include <mutex>
|
||||
#include <thread>
|
||||
|
||||
using namespace cuda::experimental::stf;
|
||||
|
||||
static __global__ void cuda_sleep_kernel(long long int clock_cnt)
|
||||
{
|
||||
long long int start_clock = clock64();
|
||||
long long int clock_offset = 0;
|
||||
while (clock_offset < clock_cnt)
|
||||
{
|
||||
clock_offset = clock64() - start_clock;
|
||||
}
|
||||
}
|
||||
|
||||
void cuda_sleep(double ms, cudaStream_t stream)
|
||||
{
|
||||
int device;
|
||||
cudaGetDevice(&device);
|
||||
|
||||
// cudaDevAttrClockRate: Peak clock frequency in kilohertz;
|
||||
int clock_rate;
|
||||
cudaDeviceGetAttribute(&clock_rate, cudaDevAttrClockRate, device);
|
||||
|
||||
long long int clock_cnt = (long long int) (ms * clock_rate);
|
||||
cuda_sleep_kernel<<<1, 1, 0, stream>>>(clock_cnt);
|
||||
}
|
||||
|
||||
__global__ void axpy(double a, slice<const double> x, slice<double> y)
|
||||
{
|
||||
int tid = blockIdx.x * blockDim.x + threadIdx.x;
|
||||
int nthreads = gridDim.x * blockDim.x;
|
||||
|
||||
for (int i = tid; i < x.size(); i += nthreads)
|
||||
{
|
||||
y(i) += a * x(i);
|
||||
}
|
||||
}
|
||||
|
||||
double X0(int i)
|
||||
{
|
||||
return sin((double) i);
|
||||
}
|
||||
|
||||
double Y0(int i)
|
||||
{
|
||||
return cos((double) i);
|
||||
}
|
||||
|
||||
void mytask(stream_ctx ctx, int /*id*/)
|
||||
{
|
||||
// std::cout << "Thread " << id << " is executing.\n";
|
||||
|
||||
const size_t N = 16;
|
||||
|
||||
double alpha = 3.14;
|
||||
|
||||
auto lX = ctx.logical_data<double>(N);
|
||||
auto lY = ctx.logical_data<double>(N);
|
||||
|
||||
ctx.task(lX.write())->*[](cudaStream_t, auto) {};
|
||||
|
||||
ctx.task(lY.write())->*[](cudaStream_t, auto) {};
|
||||
|
||||
/* Compute Y = Y + alpha X */
|
||||
for (size_t i = 0; i < 10; i++)
|
||||
{
|
||||
ctx.task(lX.read(), lY.rw())->*[&](cudaStream_t s, auto dX, auto dY) {
|
||||
axpy<<<16, 128, 0, s>>>(alpha, dX, dY);
|
||||
cuda_sleep(100.0, s);
|
||||
};
|
||||
}
|
||||
}
|
||||
|
||||
int main()
|
||||
{
|
||||
stream_ctx ctx;
|
||||
|
||||
std::vector<std::thread> threads;
|
||||
// Launch 8 threads.
|
||||
for (int i = 0; i < 10; ++i)
|
||||
{
|
||||
threads.emplace_back(mytask, ctx, i);
|
||||
}
|
||||
|
||||
// Wait for all threads to complete.
|
||||
for (auto& th : threads)
|
||||
{
|
||||
th.join();
|
||||
}
|
||||
|
||||
ctx.finalize();
|
||||
}
|
||||
Reference in New Issue
Block a user